import json
import logging
import re
import requests
from pytest_play.providers import BaseProvider


class RequestsProvider(BaseProvider):
    """ Requests command provider """

    def __init__(self, engine):
        super(RequestsProvider, self).__init__(engine)
        self.logger = logging.getLogger()

    def _make_auth(self, command):
        """ Update auth on the command """
        self.logger.warning('auth not yet implemented')

    def _make_files(self, command):
        """ Update files on the command

            requests accept parameters like that::

                files = {
                    'file': (
                        'report.xls',
                        open('report.xls', 'rb'),
                        'application/vnd.ms-excel',
                        {'Expires': '0'}
                    )
                }

            or::

                files = {
                    'file': (
                        'report.csv',
                        'some,data\nanother,row\n')}
        """
        parameters = command.get('parameters', {})
        files = parameters.get('files', {})
        if files:
            results = {}
            for key, value in files.items():
                filename = value[0]
                file_data = value[1]
                additional_args = value[2:]
                match = re.search(r'path:([^$]+)', file_data)
                if match:
                    file_path = match.group(1)
                    file_data = open(file_path, 'rb')
                if additional_args:
                    results[key] = (filename, file_data, value[2:])
                else:
                    results[key] = (filename, file_data)
            command['parameters']['files'] = results

    def _make_cookies(self, command):
        """ Update cookies on the command """
        self.logger.warning('cookies not yet implemented')

    def _merge_payload(self, command):
        """ Merge command with the default command available in
            self.engine.variables['requests']['default_payload']
        """
        play_requests = self.engine.variables.get('play_requests', {})
        if play_requests:
            default = json.loads(
                self.engine.parametrizer.parametrize(
                    json.dumps(play_requests)))
            if play_requests:
                return self._merge(default, command)
        return command

    def _merge(self, a, b, path=None):
        """ merges b and a configurations.
            Based on http://bit.ly/2uFUHgb
         """
        if path is None:
            path = []

        for key in b:
            if key in a:
                if isinstance(a[key], dict) and isinstance(b[key], dict):
                    self._merge(a[key], b[key], path + [str(key)])
                elif a[key] == b[key]:
                    pass  # same leaf value
                else:
                    # b wins
                    a[key] = b[key]
            else:
                a[key] = b[key]
        return a

    def _make_assertion(self, response, command):
        """ Make an assertion based on python
            expression against the response
        """
        assertion = command.get('assertion', None)
        if assertion:
            self.engine.execute_command(
                {'provider': 'python',
                 'type': 'assert',
                 'expression': assertion
                 },
                response=response,
            )

    def _make_variable(self, response, command):
        """ Make a variable based on python
            expression against the response
        """
        expression = command.get('variable_expression', None)
        if expression:
            self.engine.variables[
                command['variable']] = self \
                    .engine.execute_command(
                        {'provider': 'python',
                         'type': 'exec',
                         'expression': expression
                         },
                        response=response,
                    )

    def _condition(self, command):
        """ Execute a command condition
        """
        return_value = False
        condition = command.get('condition', None)
        if condition:
            return_value = self.engine.execute_command(
                {'provider': 'python',
                 'type': 'exec',
                 'expression': condition
                 }
            )
        else:
            return_value = True
        return return_value

    def _make_request(self, method, command):
        """ Make a request plus assertions """
        if self._condition(command):
            cmd = command.copy()
            url = cmd['url']

            cmd = self._merge_payload(cmd)
            self._make_auth(cmd)
            self._make_files(cmd)
            self.logger.debug('Requests call %r', cmd)
            if 'parameters' not in cmd:
                cmd['parameters'] = {}

            self.logger.debug('Effective HTTP call %r', cmd)
            response = requests.request(
                method,
                url,
                **cmd['parameters'])
            try:
                self._make_variable(response, cmd)
                self._make_assertion(response, cmd)
            except Exception as e:
                self.logger.exception(
                    'Exception for command %r',
                    cmd,
                    e)
                raise e

    def command_OPTIONS(self, command, **kwargs):
        """ OPTIONS command """
        self._make_request('OPTIONS', command)

    def command_HEAD(self, command, **kwargs):
        """ HEAD command """
        self._make_request('HEAD', command)

    def command_GET(self, command, **kwargs):
        """ GET command """
        self._make_request('GET', command)

    def command_POST(self, command, **kwargs):
        """ POST command """
        self._make_request('POST', command)

    def command_PUT(self, command, **kwargs):
        """ PUT command """
        self._make_request('PUT', command)

    def command_PATCH(self, command, **kwargs):
        """ PATCH command """
        self._make_request('PATCH', command)

    def command_DELETE(self, command, **kwargs):
        """ DELETE command """
        self._make_request('DELETE', command)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `play_requests` package."""

import os
import pytest


@pytest.fixture(scope='session')
def variables():
    return {'skins': {'skin1': {'base_url': 'http://', 'credentials': {}}}}


def test_post1(play_json):
    import requests_mock
    with requests_mock.mock() as m:
        m.request('POST',
                  'http://something/1',
                  json={'status': 'ok'})
        mock_engine = play_json
        mock_engine.variables = {}
        from play_requests import providers
        provider = providers.RequestsProvider(mock_engine)
        assert provider.engine is mock_engine
        provider.command_POST({
            'provider': 'play_requests',
            'type': 'POST',
            'url': 'http://something/1',
            'parameters': {
                'json': {
                    'foo': 'bar',
                    },
                'timeout': 2.5
                 },
        })

        history = m.request_history
        assert len(history) == 1
        assert history[0].method == 'POST'
        assert history[0].url == 'http://something/1'
        assert history[0].json() == {'foo': 'bar'}
        assert history[0].timeout == 2.5


def test_post_variables(play_json):
    import requests_mock
    with requests_mock.mock() as m:
        m.request('POST',
                  'http://something/1',
                  json={'status': 'ok'})
        mock_engine = play_json
        mock_engine.variables = {}
        from play_requests import providers
        provider = providers.RequestsProvider(mock_engine)
        assert provider.engine is mock_engine
        provider.command_POST({
            'provider': 'play_requests',
            'type': 'POST',
            'url': 'http://something/1',
            'variable': 'myvar',
            'variable_expression': 'response.json()',
            'parameters': {
                'json': {
                    'foo': 'bar',
                    },
                'timeout': 2.5
                 },
        })

        assert 'myvar' in mock_engine.variables
        assert mock_engine.variables['myvar']['status'] == 'ok'
        history = m.request_history
        assert len(history) == 1
        assert history[0].method == 'POST'
        assert history[0].url == 'http://something/1'
        assert history[0].json() == {'foo': 'bar'}
        assert history[0].timeout == 2.5


def test_post_variables_assert(play_json):
    import requests_mock
    with requests_mock.mock() as m:
        m.request('POST',
                  'http://something/1',
                  json={'status': 'ok'})
        mock_engine = play_json
        mock_engine.variables = {}
        from play_requests import providers
        provider = providers.RequestsProvider(mock_engine)
        assert provider.engine is mock_engine
        provider.command_POST({
            'provider': 'play_requests',
            'type': 'POST',
            'url': 'http://something/1',
            'variable': 'myvar',
            'variable_expression': 'response.json()',
            'assertion': 'variables["myvar"]["status"] == "ok"',
            'parameters': {
                'json': {
                    'foo': 'bar',
                    },
                'timeout': 2.5
                 },
        })

        assert 'myvar' in mock_engine.variables
        assert mock_engine.variables['myvar']['status'] == 'ok'
        history = m.request_history
        assert len(history) == 1
        assert history[0].method == 'POST'
        assert history[0].url == 'http://something/1'
        assert history[0].json() == {'foo': 'bar'}
        assert history[0].timeout == 2.5


def test_post_variables_assert_ko(play_json):
    import requests_mock
    with requests_mock.mock() as m:
        m.request('POST',
                  'http://something/1',
                  json={'status': 'KO'})
        mock_engine = play_json
        mock_engine.variables = {}
        from play_requests import providers
        provider = providers.RequestsProvider(mock_engine)
        assert provider.engine is mock_engine
        with pytest.raises(AssertionError):
            provider.command_POST({
                'provider': 'play_requests',
                'type': 'POST',
                'url': 'http://something/1',
                'variable': 'myvar',
                'variable_expression': 'response.json()',
                'assertion': 'variables["myvar"]["status"] == "ok"',
                'parameters': {
                    'json': {
                        'foo': 'bar',
                        },
                    'timeout': 2.5
                     },
            })

        assert 'myvar' in mock_engine.variables
        assert mock_engine.variables['myvar']['status'] == 'KO'
        history = m.request_history
        assert len(history) == 1
        assert history[0].method == 'POST'
        assert history[0].url == 'http://something/1'
        assert history[0].json() == {'foo': 'bar'}
        assert history[0].timeout == 2.5


@pytest.mark.parametrize('condition', [
    '1 > 0',
    '"$foo" == "SOMETHING"',
    'variables["foo"] == "SOMETHING"',
])
def test_post_condition_true(play_json, condition):
    import requests_mock
    with requests_mock.mock() as m:
        m.request('POST',
                  'http://something/1',
                  json={'status': 'ok'})
        mock_engine = play_json
        mock_engine.variables = {'foo': 'SOMETHING'}
        from play_requests import providers
        provider = providers.RequestsProvider(mock_engine)
        assert provider.engine is mock_engine
        provider.command_POST({
            'provider': 'play_requests',
            'type': 'POST',
            'url': 'http://something/1',
            'condition': condition,
            'parameters': {
                'json': {
                    'foo': 'bar',
                    },
                'timeout': 2.5
                 },
        })

        history = m.request_history
        assert len(history) == 1
        assert history[0].method == 'POST'
        assert history[0].url == 'http://something/1'
        assert history[0].json() == {'foo': 'bar'}
        assert history[0].timeout == 2.5


@pytest.mark.parametrize('condition', [
    '1 == 0',
])
def test_post_condition_false(play_json, condition):
    import requests_mock
    with requests_mock.mock() as m:
        mock_engine = play_json
        mock_engine.variables = {'foo': 'SOMETHING'}
        from play_requests import providers
        provider = providers.RequestsProvider(mock_engine)
        assert provider.engine is mock_engine
        provider.command_POST({
            'provider': 'play_requests',
            'type': 'POST',
            'url': 'http://something/1',
            'condition': condition,
            'parameters': {
                'json': {
                    'foo': 'bar',
                    },
                'timeout': 2.5
                 },
        })

        history = m.request_history
        assert len(history) == 0


def test_get(play_json):
    import requests_mock
    with requests_mock.mock() as m:
        m.request('GET',
                  'http://something/1',
                  text='OK')
        mock_engine = play_json
        mock_engine.variables = {}
        from play_requests import providers
        provider = providers.RequestsProvider(mock_engine)
        assert provider.engine is mock_engine
        provider.command_GET({
            'provider': 'play_requests',
            'type': 'GET',
            'url': 'http://something/1',
            'parameters': {
                'timeout': 2.5
                 },
        })

        history = m.request_history
        assert len(history) == 1
        assert history[0].method == 'GET'
        assert history[0].url == 'http://something/1'
        # mock requests bug
        # assert history[0].text == 'OK'
        assert history[0].timeout == 2.5


def test_no_parameters(play_json):
    import requests_mock
    with requests_mock.mock() as m:
        m.request('GET',
                  'http://something/1',
                  text='OK')
        mock_engine = play_json
        mock_engine.variables = {}
        from play_requests import providers
        provider = providers.RequestsProvider(mock_engine)
        assert provider.engine is mock_engine
        provider.command_GET({
            'provider': 'play_requests',
            'type': 'GET',
            'url': 'http://something/1',
        })

        history = m.request_history
        assert len(history) == 1
        assert history[0].method == 'GET'
        assert history[0].url == 'http://something/1'
        # mock requests bug
        # assert history[0].text == 'OK'


def test_get_params_simple(play_json):
    import requests_mock
    with requests_mock.mock() as m:
        m.request('GET',
                  'http://something/1',
                  text='OK')
        mock_engine = play_json
        mock_engine.variables = {}
        from play_requests import providers
        provider = providers.RequestsProvider(mock_engine)
        assert provider.engine is mock_engine
        provider.command_GET({
            'provider': 'play_requests',
            'type': 'GET',
            'url': 'http://something/1',
            'parameters': {
                'params': {'foo': 'bar'},
                'timeout': 2.5
                 },
        })

        history = m.request_history
        assert len(history) == 1
        assert history[0].method == 'GET'
        assert history[0].url == 'http://something/1?foo=bar'
        # mock requests bug
        # assert history[0].text == 'OK'
        assert history[0].timeout == 2.5


def test_get_params_multi(play_json):
    import requests_mock
    import re
    with requests_mock.mock() as m:
        m.request('GET',
                  'http://something/1',
                  text='OK')
        mock_engine = play_json
        mock_engine.variables = {}
        from play_requests import providers
        provider = providers.RequestsProvider(mock_engine)
        assert provider.engine is mock_engine
        provider.command_GET({
            'provider': 'play_requests',
            'type': 'GET',
            'url': 'http://something/1',
            'parameters': {
                'params': {'foo': ['bar', 'baz']},
                'timeout': 2.5
                 },
        })

        history = m.request_history
        assert len(history) == 1
        assert history[0].method == 'GET'
        match = re.search(
            r'http\://something/1\?foo=(bar|baz)&foo=(bar|baz)',
            history[0].url)
        foo1 = match.group(1)
        foo2 = match.group(2)
        assert foo1 != foo2
        assert foo1 in ('bar', 'baz')
        assert foo2 in ('bar', 'baz')
        assert history[0].timeout == 2.5


def test_post_headers(play_json):
    import requests_mock
    with requests_mock.mock() as m:
        headers = {'user-agent': 'my-app/0.0.1'}
        m.request('POST',
                  'http://something/1',
                  request_headers=headers,
                  json={'status': 'ok'})
        mock_engine = play_json
        mock_engine.variables = {}
        from play_requests import providers
        provider = providers.RequestsProvider(mock_engine)
        assert provider.engine is mock_engine
        provider.command_POST({
            'provider': 'play_requests',
            'type': 'POST',
            'url': 'http://something/1',
            'parameters': {
                'headers': headers,
                'json': {
                    'foo': 'bar',
                    },
                'timeout': 2.5
                 },
        })

        history = m.request_history
        assert len(history) == 1
        assert history[0].method == 'POST'
        assert history[0].url == 'http://something/1'
        assert history[0].json() == {'foo': 'bar'}
        assert history[0].timeout == 2.5

        # no headers
        with pytest.raises(requests_mock.exceptions.NoMockAddress):
            provider.command_POST({
                'provider': 'play_requests',
                'type': 'POST',
                'url': 'http://something/1',
                'parameters': {
                    'json': {
                        'foo': 'bar',
                        },
                    'timeout': 2.5
                     },
            })


@pytest.mark.parametrize('command', [
    {'provider': 'play_requests',
     'type': 'POST',
     'url': 'http://something/1',
     'parameters': {
         'files': {
             'filecsv': (
                 'report.csv',
                 'some,data',
                 )
             },
         },
     },
    {'provider': 'play_requests',
     'type': 'POST',
     'url': 'http://something/1',
     'parameters': {
         'files': {
             'filecsv': (
                 'report.csv',
                 'some,data',
                 'application/csv',
                 {'Expires': '0'},
                 )
             },
         },
     },
    {'provider': 'play_requests',
     'type': 'POST',
     'url': 'http://something/1',
     'parameters': {
         'files': {
             'filecsv1': (
                 'report.csv',
                 'some,data',
                 ),
             'filecsv': (
                 'report.csv',
                 'some,data',
                 'application/csv',
                 {'Expires': '0'},
                 )
             },
         },
     },
])
def test_post_files(command, play_json):
    import mock
    with mock.patch('play_requests.providers.requests.request') \
            as mock_requests:
        mock_engine = play_json
        mock_engine.variables = {}
        from play_requests import providers
        provider = providers.RequestsProvider(mock_engine)
        assert provider.engine is mock_engine
        provider.command_POST(command)

        assert mock_requests.assert_called_once_with(
            command['type'],
            command['url'],
            files=command['parameters']['files']) is None


def test_post_files_path(play_json):
    file_path = os.path.join(os.path.dirname(__file__), 'file.csv')
    command = {
         'provider': 'play_requests',
         'type': 'POST',
         'url': 'http://something/1',
         'parameters': {
             'files': {
                 'filecsv': (
                     'file.csv',
                     'path:{0}'.format(file_path),
                     )
                 },
             },
         }
    import mock
    with mock.patch('play_requests.providers.requests.request') \
            as mock_requests:
        with mock.patch('play_requests.providers.open') \
                as mock_open:
            file_mock = mock.MagicMock()
            mock_open.return_value = file_mock
            mock_engine = play_json
            mock_engine.variables = {}
            from play_requests import providers
            provider = providers.RequestsProvider(mock_engine)
            assert provider.engine is mock_engine
            provider.command_POST(command)
            assert mock_requests.assert_called_once_with(
                command['type'],
                command['url'],
                files={'filecsv': ('file.csv', file_mock)}) is None
            assert mock_open.assert_called_once_with(
                file_path, 'rb') is None


def test_post_default_parameters(play_json):
    """ If all of your calls have a set of common parameters
        you can omit them creating some defaults in the
        engine variables
    """
    import requests_mock
    with requests_mock.mock() as m:
        headers = {'user-agent': 'my-app/0.0.1'}
        m.request('POST',
                  'http://something/1',
                  request_headers=headers,
                  json={'status': 'ok'})
        mock_engine = play_json
        mock_engine.variables = {
            'play_requests': {
                'parameters': {
                    'headers': headers
                }
            }
        }

        def _parametrize(x):
            from pypom_navigation.parametrizer import Parametrizer
            parametrizer = Parametrizer(mock_engine.variables)
            return parametrizer.parametrize(x)

        mock_engine.parametrizer.parametrize = _parametrize
        from play_requests import providers
        provider = providers.RequestsProvider(mock_engine)
        assert provider.engine is mock_engine
        provider.command_POST({
            'provider': 'play_requests',
            'type': 'POST',
            'url': 'http://something/1',
            'parameters': {
                'json': {
                    'foo': 'bar',
                    },
                'timeout': 2.5
                 },
        })

        history = m.request_history
        assert len(history) == 1
        assert history[0].method == 'POST'
        assert history[0].url == 'http://something/1'
        assert history[0].json() == {'foo': 'bar'}
        assert history[0].timeout == 2.5


def test_post_default_parameters_override(play_json):
    """ If all of your calls have a set of common parameters
        you can omit them creating some defaults in the
        engine variables
    """
    import requests_mock
    with requests_mock.mock() as m:
        headers = {'user-agent': 'my-app/0.0.1'}
        m.request('POST',
                  'http://something/1',
                  request_headers={'user-agent': 'another'},
                  json={'status': 'ok'})
        mock_engine = play_json
        mock_engine.variables = {
            'play_requests': {
                'parameters': {
                    'headers': headers
                }
            }
        }

        def _parametrize(x):
            from pypom_navigation.parametrizer import Parametrizer
            parametrizer = Parametrizer(mock_engine.variables)
            return parametrizer.parametrize(x)

        mock_engine.parametrizer.parametrize = _parametrize
        from play_requests import providers
        provider = providers.RequestsProvider(mock_engine)
        assert provider.engine is mock_engine
        provider.command_POST({
            'provider': 'play_requests',
            'type': 'POST',
            'url': 'http://something/1',
            'parameters': {
                'headers': {'user-agent': 'another'},
                'json': {
                    'foo': 'bar',
                    },
                'timeout': 2.5
                 },
        })

        history = m.request_history
        assert len(history) == 1
        assert history[0].method == 'POST'
        assert history[0].url == 'http://something/1'
        assert history[0].json() == {'foo': 'bar'}
        assert history[0].timeout == 2.5


def test_post_default_parameters_override_none(play_json):
    """ If all of your calls have a set of common parameters
        you can omit them creating some defaults in the
        engine variables
    """
    import requests_mock
    with requests_mock.mock() as m:
        headers = {'user-agent': 'my-app/0.0.1'}
        m.request('POST',
                  'http://something/1',
                  request_headers={},
                  json={'status': 'ok'})
        mock_engine = play_json
        mock_engine.variables = {
            'play_requests': {
                'parameters': {
                    'headers': headers
                }
            }
        }

        def _parametrize(x):
            from pypom_navigation.parametrizer import Parametrizer
            parametrizer = Parametrizer(mock_engine.variables)
            return parametrizer.parametrize(x)

        mock_engine.parametrizer.parametrize = _parametrize
        from play_requests import providers
        provider = providers.RequestsProvider(mock_engine)
        assert provider.engine is mock_engine
        provider.command_POST({
            'provider': 'play_requests',
            'type': 'POST',
            'url': 'http://something/1',
            'parameters': {
                'headers': None,
                'json': {
                    'foo': 'bar',
                    },
                'timeout': 2.5
                 },
        })

        history = m.request_history
        assert len(history) == 1
        assert history[0].method == 'POST'
        assert history[0].url == 'http://something/1'
        assert history[0].json() == {'foo': 'bar'}
        assert history[0].timeout == 2.5


def test_post_default_parameters_parametrized(play_json):
    """ If all of your calls have a set of common parameters
        you can omit them creating some defaults in the
        engine variables
    """
    import requests_mock
    with requests_mock.mock() as m:
        m.request('POST',
                  'http://something/1',
                  request_headers={
                      'user-agent': 'my-app/0.0.1'},
                  json={'status': 'ok'})
        mock_engine = play_json
        mock_engine.variables = {
            'myapp': 'my-app',
            'play_requests': {
                'parameters': {
                    'headers': {
                        'user-agent': '$myapp/0.0.1'
                    }
                }
            }
        }

        def _parametrize(x):
            from pypom_navigation.parametrizer import Parametrizer
            parametrizer = Parametrizer(mock_engine.variables)
            return parametrizer.parametrize(x)

        mock_engine.parametrizer.parametrize = _parametrize
        from play_requests import providers
        provider = providers.RequestsProvider(mock_engine)
        assert provider.engine is mock_engine
        provider.command_POST({
            'provider': 'play_requests',
            'type': 'POST',
            'url': 'http://something/1',
            'parameters': {
                'json': {
                    'foo': 'bar',
                    },
                'timeout': 2.5
                 },
        })

        history = m.request_history
        assert len(history) == 1
        assert history[0].method == 'POST'
        assert history[0].url == 'http://something/1'
        assert history[0].json() == {'foo': 'bar'}
        assert history[0].timeout == 2.5


@pytest.mark.parametrize('assertion', [
    'response.status_code == 200',
    'response.status_code != 404',
    'response.status_code == 200 and response.json()["status"] == "ok"',
    '"status" in response.json()',
    'variables["foo"] == "baz"',
    '"foo" in variables',
    'len([1]) == 1',
    '[1][0] == 1',
    'len(list(response.json().items())) == 1',
    'variables["foo"].upper() == "BAZ"',
    'match(r"^([0-9]*)-data", "123-data")',
    'match(r"^([0-9]*)-data", "123-data").group(1) == "123"'
])
def test_post_assertion(assertion, play_json):
    import requests_mock
    with requests_mock.mock() as m:
        m.request('POST',
                  'http://something/1',
                  json={'status': 'ok'})
        mock_engine = play_json
        mock_engine.variables = {'foo': 'baz'}
        from play_requests import providers
        provider = providers.RequestsProvider(mock_engine)
        assert provider.engine is mock_engine
        provider.command_POST({
            'provider': 'play_requests',
            'type': 'POST',
            'url': 'http://something/1',
            'parameters': {
                'json': {
                    'foo': 'bar',
                    },
                'timeout': 2.5
                 },
            'assertion': assertion
        })

        history = m.request_history
        assert len(history) == 1
        assert history[0].method == 'POST'
        assert history[0].url == 'http://something/1'
        assert history[0].json() == {'foo': 'bar'}
        assert history[0].timeout == 2.5


def test_post_assertion_ko(play_json):
    import requests_mock
    with requests_mock.mock() as m:
        m.request('POST',
                  'http://something/1',
                  json={'status': 'ok'})
        mock_engine = play_json
        mock_engine.variables = {'foo': 'baz'}
        from play_requests import providers
        provider = providers.RequestsProvider(mock_engine)
        assert provider.engine is mock_engine
        with pytest.raises(AssertionError):
            provider.command_POST({
                'provider': 'play_requests',
                'type': 'POST',
                'url': 'http://something/1',
                'parameters': {
                    'json': {
                        'foo': 'bar',
                        },
                    'timeout': 2.5
                     },
                'assertion': 'response.status_code == 404'
            })

        history = m.request_history
        assert len(history) == 1
        assert history[0].method == 'POST'
        assert history[0].url == 'http://something/1'
        assert history[0].json() == {'foo': 'bar'}
        assert history[0].timeout == 2.5


@pytest.mark.parametrize('assertion', [
    'open("/etc/passwd", "r")',
    'open',
    'import os',
    '__file__',
    '__file__',
    '__builtins__.__dict__["bytes"]',
    '__builtins__.__dict__["bytes"] = "pluto"',
    'prova = lambda: 1',
    'os = 1',
])
def test_post_assertion_bad(assertion, play_json):
    import requests_mock
    with requests_mock.mock() as m:
        m.request('POST',
                  'http://something/1',
                  json={'status': 'ok'})
        mock_engine = play_json
        mock_engine.variables = {'foo': 'baz'}
        from play_requests import providers
        provider = providers.RequestsProvider(mock_engine)
        assert provider.engine is mock_engine
        with pytest.raises(Exception):
            provider.command_POST({
                'provider': 'play_requests',
                'type': 'POST',
                'url': 'http://something/1',
                'parameters': {
                    'json': {
                        'foo': 'bar',
                        },
                    'timeout': 2.5
                     },
                'assertion': assertion
            })

        history = m.request_history
        assert len(history) == 1
        assert history[0].method == 'POST'
        assert history[0].url == 'http://something/1'
        assert history[0].json() == {'foo': 'bar'}
        assert history[0].timeout == 2.5


@pytest.mark.parametrize('verb', [
    'OPTIONS',
    'HEAD',
    'PUT',
    'PATCH',
    'DELETE',
])
def test_other_verbs(verb, play_json):
    """ """
    import mock
    _make_request = mock.MagicMock()
    from play_requests import providers
    provider = providers.RequestsProvider(play_json)
    provider._make_request = _make_request
    command = {'provider': 'play_requests', 'type': verb}
    getattr(provider, 'command_{0}'.format(verb))(command, foo='bar')
    assert _make_request.assert_called_once_with(verb, command) is None
